//-----------------------------------------------------------------------------
// File: fcgrasp.cc      
//
// Purpose: Structure-aware SAT solver. 
//
// Remarks: --
//
// History: 05/13/99 - LGS - created
//
// Copyright (c) 1999 ALGOS Group / INESC
//-----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h> 
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "utils/time.hh"
#include "utils/random.hh"
#include "utils/memory.hh"
#include "utils/switches.hh"

#include "fcgrp_Defines.hh"
#include "fcgrp_CDB.hh"
#include "fcgrp_Mode.hh"
#include "fcgrp_SAT.hh"
#include "fcgrp_Circuit.hh"

extern void register_switches (SwitchHandler &sw_handler);
extern void configure_options (SwitchHandler &sw_handler);
extern void initialize();

enum TopLevelModes { _VERBOSITY_=0x00, _RUN_MODE_=0x01, _DELAY_MODEL_=0x02,
                     _STEP_STRATEGY_ = 0x03 };

Array<int> _mode;

//-----------------------------------------------------------------------------
// Function: main
//
// Purpose: Top-level actions of tau-grasp.
//-----------------------------------------------------------------------------

main( int argc, char *argv[] )
{
    char *fname;
    NodePtr node;
    register int i, j;

    Timer time;
    SwitchHandler sw_handler;
    ModeOptions mode;
    ClauseDatabase clDB(mode);
    Circuit circuit (clDB);
    GRASP sat_solver (mode, sw_handler, clDB, circuit);
    int status;

    printf ("\nCGRASP - Circuit-aware GRASP\n");
    printf ("Copyright (c) 1999 ALGOS Group / INESC\n\n");

    if (argc == 1) {
      printf ("  Usage: %s [GRASP switches] <extended ISCAS'89 file>\n\n",
              argv[0]);
      exit(0);
    }

    if( fname = sw_handler.parse (argc, argv) ) {
        DBG1(cout<<"Filename: "<<fname<<endl;);
        configure_options (sw_handler);
    }
    else
      exit(1);

    time.presetTime();
    printf ("Solving %s...\n", fname);
    circuit.load_netlist_iscas85 (fname);
    sat_solver.initialize();
    sat_solver.setup();

    status = sat_solver.solve();
    if (status == SATISFIABLE)
      printf ("Done searching... SATISFIABLE INSTANCE\n");
    else if (status == UNSATISFIABLE)
      printf ("Done searching... UNSATISFIABLE INSTANCE\n");
    else if (status == ABORTED)
      printf ("Resources exceeded... ABORTED\n");
    else 
      printf ("Abnormal termination.\n");
    sat_solver.output_stats();
    printf ("Elapsed Time: %.3fs\n\n", time.elapsedTime());
}

void register_switches (SwitchHandler &sw_handler)
{

}

void configure_options (SwitchHandler &sw_handler)
{
  SwitchMap *sw_map;

  _mode.resize(4);

  sw_map = sw_handler.switch_arg ("V");
  if (sw_map->active())
    _mode[_VERBOSITY_] = (int) (*sw_map);

}
